 package com.inet.ailink.receiver.service.impl;

import java.io.File;
import java.io.IOException;
import java.math.BigDecimal;
import java.nio.charset.StandardCharsets;
import java.util.*;
import javax.servlet.http.HttpServletRequest;

import com.inet.ailink.receiver.common.utils.*;
import org.springframework.stereotype.Service;
import com.inet.ailink.receiver.common.enums.StatusCode;
import com.inet.ailink.receiver.common.vo.Response;
import com.inet.ailink.receiver.entity.SysLogs;

 @Service
public class SysLogsServiceImpl
{
	
	String logPath = "-log.log";

	String productKey = "FjPkkwq902nAodaC";
	String productSecret = "kPO5mod7mi9jhobgabvopqinkAnujh";
	
	public Response<Object> save(String params, HttpServletRequest request) throws IOException {
		Response<Object> result = new Response<Object>();
		String deviceDataEncrypt = "";
		String requestSign = "";
		Map<String,Object> deviceDataMap = new HashMap<>();
		try {
			//将请求参数params转换为map对象
			Map<String,String> paramsMap = JsonUtils.fromJson(params, Map.class);
			//获取请求参数params的值，即，本次推送的设备测量结果的加密数据
			deviceDataEncrypt = paramsMap.get("params");
			//获取请求头信息中的productKey，通过productKey，获取productSecret
//			String productKey = request.getHeader("productKey");
//			String productSecret = "";
			//解析数据
			//1.解密数据，使用AES/CBC/PKCS5Padding进行解密
			String deviceDataDecrypt = AESUtils.aesCBCDecrypt(deviceDataEncrypt,productSecret,productKey);
			//2.校验签名
			deviceDataMap = JsonUtils.fromJson(deviceDataDecrypt,Map.class);
			String sign = MD5Util.generateSignature(deviceDataMap,productSecret);
			requestSign = request.getHeader("sign");
			if(!sign.equals(requestSign)){
				//处理签名不一致的情况
			}
			//3.消息去重,同一条数据的requestId相同，可以根据requestId进行数据去重
			String requestId = deviceDataMap.get("requestId").toString();
			//4.业务处理，依据数据类型CID，区分不同类型的设备数据
			String cid = deviceDataMap.get("cid").toString();
			if(cid.equals("80")){//噪音计的CID=80
				saveNoiseData(deviceDataMap);
			}
		} catch (Exception e) {
			result.setStatus(StatusCode.FAIL.getCode());
			result.setMsg(StatusCode.FAIL.getMsg());
			//保存日志记录
			saveLog(e,deviceDataEncrypt,request,deviceDataMap,requestSign,result);
			return  result;
		}
		result.setStatus(StatusCode.SUCCESS.getCode());
		result.setMsg(StatusCode.SUCCESS.getMsg());
		//测试返回错误信息，重新推送
		int a = (int) (Math.random()*10);
		if(a == 1 || a == 3 || a == 5){
			result.setStatus(StatusCode.FAIL.getCode());
			result.setMsg(StatusCode.FAIL.getMsg());
		}
		//保存日志记录
		saveLog(null,deviceDataEncrypt,request,deviceDataMap,requestSign,result);
		return result;
		
	}

	/**
	 * 保存噪音计数据
	 * @param deviceDataMap
	 */
	public void saveNoiseData(Map<String,Object> deviceDataMap ){
		//持久化数据或者进行消息通知等业务处理
	}


	/**
	 * 保存请求的日志
	 * @param e
	 * @param deviceDataEncrypt
	 * @param request
	 * @param deviceDataMap
	 * @param requestSign
	 * @param result
	 */
	public void saveLog(Exception e,String deviceDataEncrypt,HttpServletRequest request,Map<String,Object> deviceDataMap,String requestSign,Response<Object> result){
		//保存日志记录
		SysLogs log = new SysLogs();
		log.setEncryptParams(deviceDataEncrypt);
		log.setIp(IpUtils.getIpAddr(request));
		log.setUri(request.getRequestURI());
		log.setCreateTime(DateUtils.getStringTodayB());
		log.setSortCloumns(DateUtils.getCurTime());
		log.setDecryptParams(deviceDataMap);
		log.setProductKey(productKey);
		log.setSign(requestSign);
		log.setResult(result);
		if(e!= null){
			log.setErrorMsg(e.getMessage());
		}
		try {
			TxtUtils.string2Txt(JsonUtils.toJson(log)+"\n",DateUtils.format(new Date(),DateUtils.DATE_FORMAT_PATTERN_NO_SEPARATOR)+logPath);
		} catch (IOException ex) {

		}
	}

	/**
	 * 获取保存的日志记录数据
	 * @param date
	 * @return
	 */
	public Object getData(String date) {
		if(StringSelfUtil.isEmpty(date)){
			date = DateUtils.format(new Date(),DateUtils.DATE_FORMAT_PATTERN_NO_SEPARATOR);
		}
		List<String> strList = TxtUtils.txt2String(new File(date+logPath));
		List<SysLogs> resultList = new ArrayList<SysLogs>();
		for(String obj:strList){
			resultList.add(JsonUtils.fromJson(obj, SysLogs.class));
		}
		Collections.sort(resultList);
		return resultList;
	}

	
}